import re 

def arStrip(input_str , diacs=True , smallDiacs=False , shaddah=False ,  digit=False, alif=False , specialChars=False ):
    
    """
    This function removes Arabic diacritics, Quranic annotation signs, shaddah, English and Arabic digits, unify alif with hamzah,
    some special characters, spaces, underscore and Arabic tatwelah from the input string.

    Args:
        - input_str (:obj:`str`): Arabic text to be processed.
        - diacs (:obj:`bool`): flag to remove Arabic diacretics [ ًٌٍَُِْ] (default is True).
        - smallDiacs (:obj:`bool`): flag to remove small Quranic annotation signs (default is False).
        - shaddah (:obj:`bool`): flag to remove shaddah (default is False).
        - digit (:obj:`bool`): flag to remove English and Arabic digits (default is False).
        - alif (:obj:`bool`): flag to unify alif with hamzah (default is False).
        - specialChars (:obj:`bool`): flag to remove some special characters (default is False).

    Returns:
        - :obj:`str`: processed string with removed diacritics, Quranic annotation signs, shaddah, digits, and special characters.

    Raises:
    None.
    """
    try:
        if input_str: # if the input string is not empty do the following
            #print("in if")
            if diacs == True :
                input_str = re.sub(r'[\u064B-\u0650]+', '',input_str) # Remove all Arabic diacretics [ ًٌٍَُِْ]
                input_str = re.sub(r'[\u0652]+', '',input_str) # Remove SUKUN
            if shaddah == True:
                input_str = re.sub(r'[\u0651]+', '',input_str) # Remove shddah
            if smallDiacs == True:
                input_str = re.sub(r'[\u06D6-\u06ED]+', '',input_str) # Remove all small Quranic annotation signs
            if digit == True:
                input_str = re.sub('[0-9]+', ' ',input_str) # Remove English digits
                input_str = re.sub('[٠-٩]+', ' ',input_str)# Remove Arabic digits
            
            if alif == True:                             # Unify alif with hamzah: 
                input_str = re.sub('ٱ', 'ا',input_str);
                input_str = re.sub('أ', 'ا',input_str);
                input_str = re.sub('إ', 'ا',input_str);
                input_str = re.sub('آ', 'ا',input_str);
            if specialChars == True:
                input_str = re.sub('[?؟!@#$%-]+' , '' , input_str) # Remove some of special chars 

            input_str = re.sub('[\\s]+'," ",input_str) # Remove all spaces
            input_str = input_str.replace("_" , '') #Remove underscore
            input_str = input_str.replace("ـ" , '') # Remove Arabic tatwelah
            input_str = input_str.strip() # Trim input string
    except:
        return input_str
    return input_str
    
def ArabicStrip(input_str, diacs=True, smallDiacs=False, shaddah=False, digit=False, alif=False, specialChars=False):
    """
    Removes diacritics, small Quranic annotation signs, shaddah, English and Arabic digits, alif, special characters, spaces,
    underscore, and Arabic tatwelah from the input string.

    Args:
        - input_str (:obj:`str'): A string to be processed.
        - diacs (:obj:`bool'): A Boolean value indicating whether to remove diacritics or not.
        - smallDiacs (:obj:`bool'): A Boolean value indicating whether to remove small Quranic annotation signs or not.
        - shaddah (:obj:`bool'): A Boolean value indicating whether to remove shaddah or not.
        - digit (:obj:`bool'): A Boolean value indicating whether to remove English and Arabic digits or not.
        - alif (:obj:`bool'): A Boolean value indicating whether to unify alif with hamzah or not.
        - specialChars (:obj:`bool'): A Boolean value indicating whether to remove some special characters or not.
    
    Returns:
        - A processed (:obj:`str') without diacritics, small Quranic annotation signs, shaddah, English and Arabic digits,
             alif, special characters, spaces, underscore, and Arabic tatwelah.
    """

    if not input_str:  # if the input string is empty, return it as is
        return input_str

    # Remove diacritics
    if diacs:
        input_str = re.sub(r'[\u064B-\u0650]+', '', input_str)  # Remove all Arabic diacretics [ ًٌٍَُِْ]
        input_str = re.sub(r'[\u0652]+', '', input_str)  # Remove SUKUN

    # Remove shaddah
    if shaddah:
        input_str = re.sub(r'[\u0651]+', '', input_str)

    # Remove small Quranic annotation signs
    if smallDiacs:
        input_str = re.sub(r'[\u06D6-\u06ED]+', '', input_str)

    # Remove English and Arabic digits
    if digit:
        input_str = re.sub('[0-9٠-٩]+', ' ', input_str)

    # Unify alif with hamzah
    if alif:
        input_str = re.sub('[ٱأإآ]', 'ا', input_str)

    # Remove some of special chars
    if specialChars:
        input_str = re.sub('[?؟!@#$%-]+', '', input_str)

    # Remove all spaces, underscore, and Arabic tatwelah
    input_str = re.sub('[\\s_ـ]+', "", input_str)

    # Trim input string
    input_str = input_str.strip()

    return input_str
   
def removePunctuation( inputString ):
    """
    Removes punctuation marks from the input string.
    
    Args:
      - inputString (:obj:`str`): The input string containing punctuation marks.
    
    Returns:
      - (:obj:`str`): The output string with all punctuation marks removed.
    """
    outputString = inputString
    try:
        if inputString:
            # English Punctuation
            outputString = re.sub(r'[\u0021-\u002F]+', '',inputString) # ! " # $ % & ' ( ) * + ,  - . /
            outputString = re.sub(r'[U+060C]+', '',outputString) # ! " # $ % & ' ( ) * + ,  - . /
            outputString = re.sub(r'[\u003A-\u0040]+', '',outputString) # : ; < = > ? @ 
            outputString = re.sub(r'[\u005B-\u0060]+', '',outputString) # [ \ ] ^ _ `
            outputString = re.sub(r'[\u007B-\u007E]+', '',outputString) # { | } ~
            # Arabic Punctuation
            outputString = re.sub(r'[\u060C]+', '',outputString) # ،
            outputString = re.sub(r'[\u061B]+', '',outputString) # ؛
            outputString = re.sub(r'[\u061E]+', '',outputString) # ؞
            outputString = re.sub(r'[\u061F]+', '',outputString) # ؟
            outputString = re.sub(r'[\u0640]+', '',outputString) # ـ
            outputString = re.sub(r'[\u0653]+', '',outputString) # ٓ
            outputString = re.sub(r'[\u065C]+', '',outputString) #  ٬
            outputString = re.sub(r'[\u066C]+', '',outputString) #  ٜ 
            outputString = re.sub(r'[\u066A]+', '',outputString) # ٪
            outputString = re.sub(r'["}"]+', '',outputString) 
            outputString = re.sub(r'["{"]+', '',outputString) 
            # outputString = re.sub(r'[\u066B]+', '',outputString) # ٫ 
            # outputString = re.sub(r'[\u066D ]+','',outputString) # ٭
            # outputString = re.sub(r'[\u06D4 ]+','',outputString) # ۔
    except:
        return inputString
    # print(outputString)
    return outputString
def remove_punctuation(inputString):
    """
    Removes punctuation marks from the input string.
    
    Args:
      - inputString (:obj:`str`): The input string containing punctuation marks.
    
    Returns:
      - (:obj:`str`): The output string with all punctuation marks removed.
    """
    try:
        if inputString:
            punctuation_marks = [r'[\u0021-\u002F]+', r'[U+060C]+', r'[\u003A-\u0040]+',
                                 r'[\u005B-\u0060]+', r'[\u007B-\u007E]+', r'[\u060C]+',
                                 r'[\u061B]+', r'[\u061E]+', r'[\u061F]+', r'[\u0640]+',
                                 r'[\u0653]+', r'[\u065C]+', r'[\u066C]+', r'[\u066A]+',
                                 r'["}"]+', r'["{"]+']
            outputString = inputString
            for p in punctuation_marks:
                outputString = re.sub(p, '', outputString)
    except:
        return inputString
    return outputString

def removeEnglish( inputString ):
    """
    Removes all English characters from the input string.

    Args:
         - inputString (:obj: `str`): The string to remove English characters from.

    Returns:
         - outputString (:obj: `str`): The input string with all English characters removed.
        If an error occurs during processing, the original input string is returned.
    """
    try:
        if inputString:
            inputString = re.sub('[a-zA-Z]+', ' ',inputString)
    except:
        return inputString
    return inputString

def remove_latin(input_string):
    """
    Removes all English characters from the input string.

    Args:
         - inputString (:obj: `str`): The string to remove English characters from.

    Returns:
         - outputString (:obj: `str`): The input string with all English characters removed.
        If an error occurs during processing, the original input string is returned.
    """
    try:
        if input_string:
            input_string = re.sub('[a-zA-Z]+', ' ', input_string)
    except:
        return input_string
    return input_string
# print(removeEnglish("miojkdujhvaj1546545spkdpoqfoiehwv nWEQFGWERHERTJETAWIKUYFC"))
# print(removePunctuation("te!@#،$%%؟st") )
# Example
# print(arStrip( " مَحًمٌٍُِ" ,True ,True ,True ,True ,False , True ))




import logging
import torch
from torch.utils.data import Dataset
from torch.nn.utils.rnn import pad_sequence
from nlptools.arabiner.transforms import (
    BertSeqTransform,
    NestedTagsTransform
)

logger = logging.getLogger(__name__)


class Token:
    def __init__(self, text=None, pred_tag=None, gold_tag=None):
        """
        Token object to hold token attributes
        :param text: str
        :param pred_tag: str
        :param gold_tag: str
        """
        self.text = text
        self.gold_tag = gold_tag
        self.pred_tag = pred_tag
        self.subwords = None

    @property
    def subwords(self):
        return self._subwords

    @subwords.setter
    def subwords(self, value):
        self._subwords = value

    def __str__(self):
        """
        Token text representation
        :return: str
        """
        gold_tags = "|".join(self.gold_tag)
        pred_tags = "|".join([pred_tag["tag"] for pred_tag in self.pred_tag])

        if self.gold_tag:
            r = f"{self.text}\t{gold_tags}\t{pred_tags}"
        else:
            r = f"{self.text}\t{pred_tags}"

        return r


class DefaultDataset(Dataset):
    def __init__(
        self,
        examples=None,
        vocab=None,
        bert_model="aubmindlab/bert-base-arabertv2",
        max_seq_len=512,
    ):
        """
        The dataset that used to transform the segments into training data
        :param examples: list[[tuple]] - [[(token, tag), (token, tag), ...], [(token, tag), ...]]
                         You can get generate examples from -- arabiner.data.dataset.parse_conll_files
        :param vocab: vocab object containing indexed tags and tokens
        :param bert_model: str - BERT model
        :param: int - maximum sequence length
        """
        self.transform = BertSeqTransform(bert_model, vocab, max_seq_len=max_seq_len)
        self.examples = examples
        self.vocab = vocab

    def __len__(self):
        return len(self.examples)

    def __getitem__(self, item):
        subwords, tags, tokens, valid_len = self.transform(self.examples[item])
        return subwords, tags, tokens, valid_len

    def collate_fn(self, batch):
        """
        Collate function that is called when the batch is called by the trainer
        :param batch: Dataloader batch
        :return: Same output as the __getitem__ function
        """
        subwords, tags, tokens, valid_len = zip(*batch)

        # Pad sequences in this batch
        # subwords and tokens are padded with zeros
        # tags are padding with the index of the O tag
        subwords = pad_sequence(subwords, batch_first=True, padding_value=0)
        tags = pad_sequence(
            tags, batch_first=True, padding_value=self.vocab.tags[0].get_stoi()["O"]
        )
        return subwords, tags, tokens, valid_len


class NestedTagsDataset(Dataset):
    def __init__(
        self,
        examples=None,
        vocab=None,
        bert_model="aubmindlab/bert-base-arabertv2",
        max_seq_len=512,
    ):
        """
        The dataset that used to transform the segments into training data
        :param examples: list[[tuple]] - [[(token, tag), (token, tag), ...], [(token, tag), ...]]
                         You can get generate examples from -- arabiner.data.dataset.parse_conll_files
        :param vocab: vocab object containing indexed tags and tokens
        :param bert_model: str - BERT model
        :param: int - maximum sequence length
        """
        self.transform = NestedTagsTransform(
            bert_model, vocab, max_seq_len=max_seq_len
        )
        self.examples = examples
        self.vocab = vocab

    def __len__(self):
        return len(self.examples)

    def __getitem__(self, item):
        subwords, tags, tokens, masks, valid_len = self.transform(self.examples[item])
        return subwords, tags, tokens, masks, valid_len

    def collate_fn(self, batch):
        """
        Collate function that is called when the batch is called by the trainer
        :param batch: Dataloader batch
        :return: Same output as the __getitem__ function
        """
        subwords, tags, tokens, masks, valid_len = zip(*batch)

        # Pad sequences in this batch
        # subwords and tokens are padded with zeros
        # tags are padding with the index of the O tag
        subwords = pad_sequence(subwords, batch_first=True, padding_value=0)

        masks = [torch.nn.ConstantPad1d((0, subwords.shape[-1] - tag.shape[-1]), 0)(mask)
                 for tag, mask in zip(tags, masks)]
        masks = torch.cat(masks)

        # Pad the tags, do the padding for each tag type
        tags = [torch.nn.ConstantPad1d((0, subwords.shape[-1] - tag.shape[-1]), vocab.get_stoi()["<pad>"])(tag)
                for tag, vocab in zip(tags, self.vocab.tags[1:])]
        tags = torch.cat(tags)

        return subwords, tags, tokens, masks, valid_len


# -*- coding: utf-8 -*-
"""
Created on Wed Dec  1 14:22:26 2021

@author: Tymaa
"""

from nlptools.utils.parser import arStrip
from nlptools.utils.implication import Implication


def normalize_word(word: str, ignore_all_diacritics_but_not_shadda: bool, ignore_shadda_diacritic: bool) -> str:
    """
    Normalize a given Arabic word by removing diacritics and/or shadda diacritic based on the provided flags.

    Args:
        word: A string representing an Arabic word to be normalized.
        ignore_all_diacritics_but_not_shadda: A boolean flag indicating whether to remove all diacritics except shadda.
        ignore_shadda_diacritic: A boolean flag indicating whether to remove shadda diacritic.

    Returns:
        A string representing the normalized Arabic word.
    """
    if ignore_all_diacritics_but_not_shadda:
        # Remove all diacritics except shadda
        word = arStrip(word, True, True, False, False, False, False)
        
    if ignore_shadda_diacritic:
        # Remove shadda diacritic
        word = arStrip(word, False, False, True, False, False, False)
    
    return word

    
def get_preferred_word(word1, word2):
    """
    Returns the preferred word among two given words based on their implication.

    Args:
        word1 (:obj:str): The first word to be compared.
        word2 (:obj:str): The second word to be compared.

    Returns:
        str: The preferred word among the two given words.
    """
    implication = Implication(word1, word2)
    
    direction = implication.getDirection()
    
    if direction in (0, 2):
        return word1
       
    elif direction == 1:
        return word2
       
    elif direction == 3:
        if not word1.endswith("َ") and not word1.endswith("ُ"):
            return word2
        return word1

           
    
def get_non_preferred_word(word1, word2):
    """
    Find the non-preferred word between the two input words.

    Args:
        word1 (:obj:str): The first word.
        word2 (:obj:str): The second word.

    Returns:
        str: The non-preferred word. If there is no non-preferred word, returns '#'.

    """
    # Find non-preferred word if the distance between the two words input is less than 15.
    implication = Implication(word1, word2)
    if implication.getDistance() < 15:
        direction = implication.getDirection()
        if direction == 0 or direction == 1:
            return word1
        elif direction == 2:
            return word2
        elif direction == 3:
            if not word1.endswith("َ") and not word1.endswith("ُ"):
                return word1
            return word2
    return "#"

def get_intersection(list1, list2, ignore_all_diacritics_but_not_shadda=False, ignore_shadda_diacritic=False):
    """
    Get the intersection of two lists after normalization and ignoring diacritics based on input flags.

    Args:
        - list1 (:obj:`list`): The first list.
        - list2 (:obj:`list`): The second list.
        - ignore_all_diacritics_but_not_shadda (:obj:`bool`, optional): A flag to ignore all diacritics except for the shadda. Defaults to False.
        - ignore_shadda_diacritic (:obj:`bool`, optional): A flag to ignore the shadda diacritic. Defaults to False.

    Returns:
        - (:obj:`list`): The intersection of the two lists after normalization and ignoring diacritics.

    """

    # Remove all None and empty values from first list
    list1 = [str(i) for i in list1 if i not in (None, ' ', '')]
    list1 = [str(i.strip()) for i in list1]

    # Remove all None and empty values from second list
    list2 = [str(i) for i in list2 if i not in (None, ' ', '')]
    list2 = [str(i.strip()) for i in list2]

    interection_list = []

    # Add all Common words between the two list1 and list2 to interectionList
    for list1_word in list1:
        for list2_word in list2:
            word1 = normalize_word(list1_word, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)
            word2 = normalize_word(list2_word, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)

            implication = Implication(word1, word2)
            if implication.get_direction() >= 0 and implication.get_distance() < 15:
                interection_list.append(get_preferred_word(word1, word2))

    i = 0
    while i < len(interection_list):
        j = i + 1
        while j < len(interection_list):
            non_preferred_word = get_non_preferred_word(interection_list[i], interection_list[j])
            if non_preferred_word != "#":
                interection_list.remove(non_preferred_word)
            j += 1
        i += 1

    return interection_list
             


def get_union(list1, list2, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic):
    """
    Finds the union of two lists by removing duplicates and normalizing words.

    Args:
        - list1 (:obj:`list`): The first list.
        - list2 (:obj:`list`): The second list.
        - ignore_all_diacritics_but_not_shadda (:obj:`bool`): Whether to ignore all diacritics except shadda or not.
        - ignore_shadda_diacritic (:obj:`bool`): Whether to ignore shadda diacritic or not.

    Returns:
        - :obj:`list`: The union of the two lists after removing duplicates and normalizing words.
    """
    # Remove all None and empty values from the first set
    list1 = [str(i) for i in list1 if i not in (None, ' ', '')]

    # Remove all None and empty values from the second set
    list2 = [str(i) for i in list2 if i not in (None, ' ', '')]

    union_list = []

    # Add all words found in set#1 to union_list after normalization
    for list1_word in list1:
        word1 = normalize_word(list1_word, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)
        union_list.append(word1)

    # Add all words found in set#2 to union_list after normalization
    for list2_word in list2:
        word2 = normalize_word(list2_word, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)
        union_list.append(word2)

    # Remove redundant words by using get_non_preferred_word function
    i = 0
    while i < len(union_list):
        j = i + 1
        while j < len(union_list):
            non_preferred_word = get_non_preferred_word(union_list[i], union_list[j])
            if (non_preferred_word != "#"):
                union_list.remove(non_preferred_word)
            j = j + 1
        i = i + 1

    return union_list
      


def jaccard_similarity(list1: list, list2: list, ignore_all_diacritics_but_not_shadda: bool, ignore_shadda_diacritic: bool) -> float:
    """
    Calculates the Jaccard similarity coefficient between two lists.

    Args:
        - list1 (:obj:`list`): The first list.
        - list2 (:obj:`list`): The second list.
        - ignore_all_diacritics_but_not_shadda (:obj:`bool`): A flag indicating whether to ignore all diacritics except for shadda.
        - ignore_shadda_diacritic (:obj:`bool`): A flag indicating whether to ignore the shadda diacritic.

    Returns:
        float: The Jaccard similarity coefficient between the two lists.
    """
    # Find the intersection between two sets
    intersection_list = get_intersection(list1, list2, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)
    
    # Find the union between two sets
    union_list = get_union(list1, list2, ignore_all_diacritics_but_not_shadda, ignore_shadda_diacritic)
    
    # Calculate the Jaccard similarity coefficient by dividing the length of the intersectionList by the length of the unionList
    return float(len(intersection_list)) / float(len(union_list))
   



def jaccard(delimiter, str1, str2, selection, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic):
    """
    Compute the Jaccard similarity, union, or intersection of two sets of strings.

    Args:
        delimiter (:obj:`str`): The delimiter used to split the input strings.
        str1 (:obj:`str`): The first input string to compare.
        str2 (:obj:`str`): The second input string to compare.
        selection (:obj:`str`): The desired operation to perform on the two sets of strings. 
                         Must be one of "intersection", "union", "jaccardSimilarity", or "jaccardAll".
        ignoreAllDiacriticsButNotShadda (:obj:`bool`): If True, ignore all diacritics except for the Shadda diacritic.
        ignoreShaddaDiacritic (:obj:`bool`): If True, ignore the Shadda diacritic.

    Returns:
        The Jaccard similarity, union, or intersection of the two sets of strings, 
        depending on the value of the `selection` argument.

        If `selection` is "jaccardAll", a list of the intersection, union, and Jaccard similarity 
        of the two sets of strings will be returned.

        If an error occurs, the method will return the string "An error has occurred".
    """
    try:
        list1 = str1.split(delimiter)
        list2 = str2.split(delimiter)

        if selection == "intersection":
            intersection = get_intersection(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            return intersection
        elif selection == "union":
            union = get_union(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            return union
        elif selection == "jaccardSimilarity":      
            similarity = jaccard_similarity(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            return similarity
        elif selection == "jaccardAll":    
            intersection = get_intersection(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            union = get_union(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            similarity = jaccard_similarity(list1, list2, ignoreAllDiacriticsButNotShadda, ignoreShaddaDiacritic)
            output_list = ["intersection:", intersection, "union:", union, "similarity:", similarity]
            return output_list
        else:
            return 'Invalid selection option'

    except AttributeError as ae:
        print(f"Attribute error occurred: {str(ae)}")
        return 'Invalid input type'
    except Exception as e:
        print(f"Error occurred: {str(e)}")
        return 'An error has occurred'



#!/usr/bin/env python

"""The setup script."""
import os 
from setuptools import setup, find_packages
VERSION_FILE = os.path.join(os.path.dirname(__file__),
                            'nlptools',
                            'VERSION')
with open(VERSION_FILE, encoding='utf-8') as version_fp:
    VERSION = version_fp.read().strip()
with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'six',
    'farasapy',
    'tqdm',
    'requests',
    'regex',
    'pathlib',
    # 'torch==1.13.0',
    # 'transformers==4.24.0',
    # 'torchtext==0.14.0',
    # 'torchvision==0.14.0',
    # 'torchdata==0.5.1',
    # 'seqeval==1.2.2'
]

test_requirements = [ ]

setup(
    author="Alaa' Omar",
    author_email='alaa.omer2009@gmail.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Python Boilerplate contains all the boilerplate you need to create a Python package.",
    entry_points={
        'console_scripts': [
            'nlptools=nlptools.cli:main',
        ],
    },
    package_data={'nlptools': ['data/*.pickle']},
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='nlptools',
    name='nlptoolssna',
    packages=find_packages(include=['nlptools', 'nlptools.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/eng-aomar/nlptools',
    version=VERSION,
    zip_safe=False,
)
